//
//  Burgers of Hanoi
//
//  It's based strictly on the rules of Towers of Hanoi, but with
//  hamburger ingredients handled by Burly the bear.
//
//  If you have any questions regarding the code, contact me
//  via the method specified at  http://www.sebastianmihai.com
//  (operational in 2012), and I will try my best to help.
//
//                             Sebastian Mihai, 2012
//

#include <stdlib.h>
#include <sms.h>
#include "graphics.h"

#define I_NONE 0
// these are in the same order as on the burger, so that ingredient drop checks are easy
#define I_BUN_BOTTOM 1
#define I_MEAT 2
#define I_LETTUCE 3
#define I_TOMATO 4
#define I_BUN_TOP 5

#define BURGER_WIDTH 7

// Sega Master System 8x8 pixel tiles are stored in memory as such:
// 
// Each of the sixteen colours in a palette have the following bits: xxbbggrr
// where
// xx=unused, bb=blue intensity, gg=green intensity, rr=red intensity

// colors
// 0 - black (transparent)
// 1 - light green (lettuce)
// 2 - reddish brown (meat and Burly)
// 3 - light orange (buns)
// 4 - bright red (tomato)
// 5 - white (sesame seeds)
// 6 - light brown (edges of buns)
// bg palette
unsigned char pal1[] = {0x00, 0x2E, 0x06, 0x0B, 0x03, 0x3F, 0x06, 0x2B,
				0x14, 0x24, 0x39, 0x0F, 0x00, 0x15, 0x2A, 0x3F};

				
// sprites palette
unsigned char pal2[] = {0x00, 0x2E, 0x06, 0x0B, 0x03, 0x3F, 0x06, 0x2B,
				0x14, 0x24, 0x39, 0x0F, 0x00, 0x15, 0x2A, 0x3F};

// these reference tiles loaded via load_tiles
// left, middle, right (middle is repeated as many times as needed, based on BURGER_WIDTH)
unsigned int tiles_bun_bottom[] = {0x0001, 0x0002, 0x0003};
unsigned int tiles_lettuce[] = {0x0004, 0x0005, 0x0006};
unsigned int tiles_meat[] = {0x0007, 0x0008, 0x0009};
unsigned int tiles_bun_top[] = {0x000A, 0x000B, 0x000C};
unsigned int tiles_tomato[] = {0x000D, 0x000E, 0x000F};
unsigned int tiles_none[] = {0x0000, 0x0000, 0x0000};

unsigned int author_SM[] = {0x0014};
unsigned int goal[] = {0x0015};

// X positions for every slot
unsigned int slots_X[3] = { 3, 13, 23 };
#define SLOTS_Y 20
#define BURLY_Y 10

void draw_bkg() {
	//x,y,w,h	
	set_bkg_map(tiles_bun_bottom, 10, 20, 5, 1);
	set_bkg_map(tiles_meat, 10, 19, 5, 1);
	set_bkg_map(tiles_lettuce, 10, 18, 5, 1);
	set_bkg_map(tiles_tomato, 10, 17, 5, 1);
	set_bkg_map(tiles_bun_top, 10, 16, 5, 1);
}

void draw_burly_bear( int x, int y ) {
	// sprite #, x, y, tile #
	set_sprite(0, x, y, 0x10);
	set_sprite(1, x+8, y, 0x11);
	set_sprite(2, x, y+8, 0x12);
	set_sprite(3, x+8, y+8, 0x13);
}

void draw_ingredient( char which, int x, int y ) {

	unsigned int *gfx;
	int i;

	if( which == I_NONE ) {
		gfx = tiles_none;
	} else if( which == I_BUN_BOTTOM ) {
		gfx = tiles_bun_bottom;
	} else if( which == I_BUN_TOP ) {
		gfx = tiles_bun_top;
	} else if( which == I_LETTUCE) {
		gfx = tiles_lettuce;
	} else if( which == I_TOMATO ) {
		gfx = tiles_tomato;
	} else if( which == I_MEAT ) {
		gfx = tiles_meat;
	} 
	
	// left most tile
	set_bkg_map( gfx, x, y, 1, 1 );
	// middle tiles
	for(i=1;i<BURGER_WIDTH-1;i++) {
		set_bkg_map( gfx+1, x+i, y, 1, 1 );
	}
	// right tile
	set_bkg_map( gfx+2, x+BURGER_WIDTH-1, y, 1, 1 );
}

void game() {
	int joy;
	int i,j;
	int burlyPosition = 0; 
	int oldBurlyPosition;
	char burlyHolding = I_NONE;
	char lastIngredient;
	int lastIngredientPosition;
	// these are the holding slots (0-4 first, 5-10 second, 11-15 third)
	// like so
	// 4  9  14
	// 3  8  13
	// 2  7  12
	// 1  6  11
	// 0  5  10
	unsigned int slots[15];
	
	// initialize slots
	for(i=0;i<3;i++) {
		for(j=0;j<5;j++) {
			slots[i*5+j] = I_NONE;
		}
	}
	// entire burger in first slot
	slots[0] = I_BUN_BOTTOM;
	slots[1] = I_MEAT;
	slots[2] = I_LETTUCE;
	slots[3] = I_TOMATO;
	slots[4] = I_BUN_TOP;

	
	set_vdp_reg(VDP_REG_FLAGS1, VDP_REG_FLAGS1_SCREEN);

	// load all tiles defined in graphics.h
	// start, count, bpp
	load_tiles( all_tiles, 0, 22, 4);
	
	// data, index, count
	load_palette(pal1, 0, 16);
	load_palette(pal2, 16, 16);
	
	while ( 1 ) {
	
		oldBurlyPosition = burlyPosition;
		joy = read_joypad1();

		// -------------------------
		// burly pick-up and release
		// -------------------------
		if (joy & JOY_FIREA) {
			while(read_joypad1() & JOY_FIREA){}
			
			if( burlyHolding == I_NONE ) {
				// attempt pick-up
				
				lastIngredient = I_NONE;
				// look for last ingredient in the slot at burly's position
				for(j=4;j>=0;j--) {
					if( slots[burlyPosition*5+j] != I_NONE ) {
						lastIngredient = slots[burlyPosition*5+j];
						lastIngredientPosition = burlyPosition*5+j;
						break;
					}
				}
				
				// we found something, so we pick it up, and remove off slot
				if( lastIngredient != I_NONE ) {
					burlyHolding = lastIngredient;
					slots[lastIngredientPosition] = I_NONE;
				}
				
			} else {
				// attempt drop
				// bottom bun has value 1, and top has value 5, so in order to be able to
				// drop, the ingredient currently held by burly has to be greater in value than
				// the top of this slot
				lastIngredient = I_NONE;
				for(j=4;j>=0;j--) {

					if( slots[burlyPosition*5+j] != I_NONE ) {
						// we found something in this slot
						lastIngredient = slots[burlyPosition*5+j];
						
						// if it lower in value?
						if( burlyHolding > slots[burlyPosition*5+j] ) {
							// yeah, burly can drop ingredient above
							slots[burlyPosition*5+j+1] = burlyHolding;
							burlyHolding = I_NONE;
						}
						break;
					}
				}
				
				// slot was empty, so drop it at bottom of slot
				if( lastIngredient == I_NONE ) {
					// slot was empty
					slots[burlyPosition*5] = burlyHolding;
					burlyHolding = I_NONE;
				}
			}
		} else {	
			// -------------------------
			// burly movement
			// -------------------------
			if (joy & JOY_LEFT) {
				while(read_joypad1() & JOY_LEFT){}
				burlyPosition--;
			} else if (joy & JOY_RIGHT) {
				while(read_joypad1() & JOY_RIGHT){}
				burlyPosition++;
			}
			if( burlyPosition < 0 ) burlyPosition = 2;
			if( burlyPosition > 2 ) burlyPosition = 0;
		}
		
		wait_vblank_noint();
		
		// draw burly bear (all coordinates are in tile coordinates and must be converted
		// to pixel coordinates)
		draw_burly_bear( (slots_X[burlyPosition] - 2) * 8, BURLY_Y * 8 );
		// and also draw whatever he's holding
		draw_ingredient( burlyHolding, slots_X[burlyPosition], BURLY_Y + 1 );
		// erase ingredient at burly's old position, if burly moved
		if( burlyPosition != oldBurlyPosition ) {
			draw_ingredient( I_NONE, slots_X[oldBurlyPosition], BURLY_Y + 1 );
		}

		// draw slots
		for(i=0;i<3;i++) {
			for(j=0;j<5;j++) {
				draw_ingredient( slots[i*5+j], slots_X[i], SLOTS_Y - j );
			}
		}
		
		// draw author
		set_bkg_map( author_SM, 15, 5, 1, 1 );
		// draw goal
		set_bkg_map( goal, 26, SLOTS_Y + 2, 1, 1 );
		
		// check for game completion
		if( slots[14] != I_NONE )
			return;
	}
}

void main() {
	game();
	
	// lock processor
	while(1){}
}
